var net = require("net")
  , EventEmitter = require('events').EventEmitter

module.exports = ProxyRequest

ProxyRequest.prototype = Object.create(EventEmitter.prototype)

//secure option is so we can pass the same config to
//secure and insecure servers

function ProxyRequest(server){
  var that = Object.create(ProxyRequest.prototype)
  that.socket = new net.Socket()
  that.buffer = new Buffer(0)
  that.connected = false
  that.async = false

  that.on("header", function(req){
    that.getHttpHost(req, server)
  })

  that.on("tcp", function(){
    that.getTcpHost()
  })

  that.socket.on('connect', function socketConnect(){
    that.connected = true
    that.async = false
    if(that.buffer.length)
      that.socket.write(that.flushBuffer())
    that.emit("connect")
  })

  return that
}


ProxyRequest.prototype.handleClientData = function (data){
  if(this.connected === true)
    this.socket.write(data)
  else
    this.bufferData(data)
}

ProxyRequest.prototype.bufferData = function(data){
  if(data && data.length){
    var newBuffer = new Buffer(this.buffer.length + data.length)
    this.buffer.copy(newBuffer)
    data.copy(newBuffer, this.buffer.length)
    this.buffer = newBuffer
  }
}

ProxyRequest.prototype.connect = function(connection){
  if(typeof connection === 'object')
    if(connection.constructor === Array){//array, do round-robin
      var newConn = connection.shift()
      connection.push(newConn)
      this.connect(newConn)
    }
    else{
      if(connection.socketPath)
        this.socket.connect(connection.socketPath)
      else
        this.socket.connect(connection.port, connection.host)
    }
  else
    this.socket.connect(connection) //only port or domain socket
  this.socket.write(this.flushBuffer())
}

ProxyRequest.prototype.getTcpHost = function(){
  if(!this.async){
    var tcp = this.config.tcp
    if(typeof tcp === 'function'){
      this.async = true
      tcp(this.buffer, function(err, connection){
        if(err) return this.emit("error", err)
        this.connect(connection)
      }.bind(this))
    }
    else
      this.connect(tcp) 
  }
}

ProxyRequest.prototype.getHttpHost = function(req, server){
  var hosts = server.config.hosts
    , hostname = req.headers.host.split(/:/)[0]

  if(!this.async){
    if(hostname in server.cache){
      this.connect(server.config.hosts[server.cache[hostname]].connection)
    }
    else if(typeof hosts === "function"){
      this.async = true
      hosts(hostname, function(err, connection){
        if(err) return this.emit("error", err)
        this.connect(connection)
      }.bind(this))
    }
    else if(typeof hosts === 'object')//hostname matching
      Object.keys(hosts).some(function(value){
        var host = hosts[value]
        if(!host.match || host.match === hostname){
          server.cache[hostname] = value
          this.connect(host.connection)
          return true
        }
      }.bind(this)) || this.emit("error", new Error("Could not find match for host"))
    else
      this.emit("error", new Error("Hosts parameter could not be read"))
  }
}

ProxyRequest.prototype.flushBuffer = function(){
  var tmpBuffer = this.buffer
  this.buffer = new Buffer(0)
  return tmpBuffer
}
