var net = require("net")
  , tls = require("tls")
  , fs = require("fs")
  , EventEmitter = require('events').EventEmitter
  , ProxyRequest = require('./proxy-request')
  , parsers = require('http').parsers

module.exports = ProxyServer

ProxyServer.prototype = Object.create(EventEmitter.prototype)

//secure option is so we can pass the same config to
//secure and insecure servers
function ProxyServer(config, secure){
  var that = Object.create(ProxyServer.prototype)

  that.loadConfig(config, function(err, config){
    if(err) return that.emit("error", err)
    //one-time only loading from constructor
    if(secure){
      var sec = {}
      if(config.keyFile)
        sec.key = fs.readFileSync(config.keyFile)
      if(config.certFile)
        sec.cert = fs.readFileSync(config.certFile)
      that.server = new tls.createServer(sec)
    }
    else
      that.server = new net.createServer()
    that._listen()
  })
 return that
}

ProxyServer.prototype.loadConfig = function(config, cb){
  var self = this
  if(typeof config === 'object'){
    self.config = config
    self.cache = {}
    return cb(null, config)
  }
  else
    fs.readFile(config, 'utf-8', function(err, data){
      if(err) return cb(err)
      try{
        self.loadConfig(JSON.parse(data), cb)
      }catch(err){
        cb(err)
      }
    })
}

function serverConnect(clientSocket){
    var proxyRequest = new ProxyRequest(this)
    
    clientSocket.on('data', function(data){
      proxyRequest.handleClientData(data)
    })

    //simple rip off of http.js 
    var parser = parsers.alloc();
    parser.reinitialize('request');

    clientSocket.ondata = function(d, start, end) {
      var ret = parser.execute(d, start, end - start);
      if (ret instanceof Error)
        proxyRequest.emit("tcp")
      else
        proxyRequest.emit("header", {headers: parser.incoming.headers, url: parser.incoming.url, method: parser.incoming.method})
      freeParser()
    };

    parser.onIncoming = function(){
      //console.dir(arguments)
    }

    proxyRequest.socket.on('data', function(data){
      //should be writable, otherwise most likely closed and should ignore
      if(clientSocket.writable)
        clientSocket.write(data)
    })

    clientSocket.on('end', function(){
      freeParser()
      proxyRequest.socket.end()
    })

    this.emit("connect", proxyRequest) //so we can access the proxyRequest object

    function freeParser(){
      clientSocket.ondata = null
      parsers.free(parser)
      proxyRequest.socket.removeListener('end', freeParser)
    }
}

ProxyServer.prototype.listen = function(){
  this.connection = arguments
  this._listen()
}

ProxyServer.prototype._listen = function(){
  if(this.server && this.connection){
    this.server.on("connection", serverConnect.bind(this))
    this.server.listen.apply(this.server, this.connection)
  }
}
